package mactrans.database;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 * 
 * AbstractDatabase implements a few methods used by both
 * the Dictionary and the SentenceDatabase
 * 
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.4 $
 * 
 * <pre>
 * $Log: AbstractDatabase.java,v $
 * Revision 1.4  2005/01/30 15:56:11  jkieneck
 * added constructor with modulename
 *
 * Revision 1.3  2005/01/26 13:42:04  jkieneck
 * added debugging support
 *
 * Revision 1.2  2005/01/24 06:29:12  jkieneck
 * general update (too much to mention here...)
 *
 * Revision 1.1.1.1  2004/12/14 23:28:03  mactrans
 * added to repository
 *
 * </pre>
 */
public abstract class AbstractDatabase extends XMLDatabase {
	/* local vars */
	public Set data;
	
	/**
	 * Constructor for an abstract database, inits Set
	 *
	 */
	public AbstractDatabase() {
		data = new HashSet();
	}
	
	/**
	 * Constructor with XML-document (filename)
	 * 
	 * @param filename the XML-document to load data from
	 */
	public AbstractDatabase( String filename ) {
		setModuleName( filename );
		data = new HashSet();
		if ( loadXML( filename ) ) {
			// all fine
		}
	}
	
	public AbstractDatabase( String modname, String filename ) {
		setModuleName( modname );
		data = new HashSet();
		if ( loadXML( filename ) ) {
			// all fine
		}
	}
	
	public boolean loadXML( String filename ) { 
		boolean result = super.loadXML( filename );
		if ( result ) {
			debug( "Ok, " + size() + " entries loaded" );
		} else  {
			error( "Failed to load " + filename );
		}
		return result;
	}
	
	/**
	 * Use this method in order to find out whether an Id
	 * is contained within the database.
	 * 
	 * @param id a String representing a record's id
	 * @return true, if given id matches any of the records' ids.
	 */
	public boolean containsId( String id ) {
		boolean result = false;
		
		Iterator it = iterator();
		while ( !result && it.hasNext() ) {
			result = ( (AbstractRecord) it.next() ).getId().equals( id );
		}
		return result;
	}
	
	/**
	 * Use this method if you wish to extend the database. Be
	 * sure to provide an ID which is currently not contained
	 * in the database. Use containsId for this purpose.
	 * 
	 * @param r an AbstractRecord to store in the database
	 */
	public void putRecord( AbstractRecord r ) {
		if ( containsId( r.getId() ) ) {
			// remove this first
		}
		data.add( r );
	}

	/**
	 * Returns the number of records stored in the database
	 * 
	 * @return the number of records contained
	 */
	public int size() {
		return data.size();
	}

	/**
	 * Creates an iterator to walk along the database.
	 * 
	 * @return an iterator as known from Sets etc.
	 */
	public Iterator iterator() {
		return data.iterator();
	}
}
