package mactrans.database;

/**
 * 
 * AbstractRecord implements a few methods used by both
 * the DictionaryRecord and the SentenceRecord
 * 
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.1.1.1 $
 * 
 * <pre>
 * $Log: AbstractRecord.java,v $
 * Revision 1.1.1.1  2004/12/14 23:28:03  mactrans
 * added to repository
 *
 * </pre>
 */

public abstract class AbstractRecord {
	/* local vars */
	private String id;
	
	/**
	 * Creates an AbstractRecord
	 *
	 */
	public AbstractRecord() {
		id = new String();
	}
	
	/**
	 * Creates an AbstractRecord and assigns an id to it
	 * 
	 * @param id A (unique) id linked to the created record
	 */
	public AbstractRecord( String id ) {
		this.id = new String( id );
	}
	
	/**
	 * Returns the linked id
	 * 
	 * @return a String representation of the id
	 */
	public String getId() {
		return id;
	}
	
	/**
	 * Assigns an id to the record
	 * 
	 * @param id a String representing the id of this record
	 */
	public void setId( String id ) {
		this.id = id;
	}
	
	/**
	 * Two AbstractRecords are considered equal, if they have the same id
	 * 
	 * @return true, if both ids match
	 */
	public boolean equals( Object obj ) {
		boolean result = false;
		if ( obj instanceof AbstractRecord ) {
			if ( ( (AbstractRecord) obj ).getId().equals( getId() ) ) {
				result = true;
			}
		}
		return result;
	}
	
	/**
	 * Inherited hashCode from String
	 * 
	 * @return id-String's hashCode
	 */
	public int hashCode() {
		return getId().hashCode();
	}
}
