package mactrans.database;

import java.util.Iterator;
import org.w3c.dom.*;
import mactrans.tools.*;

/**
 * 
 * @author Iuliana Zibilianu (iuliacaz@yahoo.com)
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.6 $
 * 
 * <pre>
 * $Log: SentenceDatabase.java,v $
 * Revision 1.6  2005/02/02 10:27:53  jkieneck
 * so that the data will also be loaded...
 *
 * Revision 1.5  2005/01/30 15:56:46  jkieneck
 * added constructor with modulename
 *
 * Revision 1.4  2005/01/26 13:42:04  jkieneck
 * added debugging support
 *
 * Revision 1.3  2005/01/24 06:29:12  jkieneck
 * general update (too much to mention here...)
 *
 * Revision 1.2  2005/01/05 13:30:29  jkieneck
 * added required method getTokensById, which copies a partial string from a db-sentence
 *
 * Revision 1.1.1.1  2004/12/14 23:28:03  mactrans
 * added to repository
 *
 * </pre>
 */
public class SentenceDatabase extends AbstractDatabase {
	public SentenceDatabase( String filename ) {
		super( filename );
	}
	
	public SentenceDatabase( String modname, String filename ) {
		super( modname, filename );
	}
	
	private void addRecord( Node tNode ) {
		//debug( "entered addRecord with node '" + tNode.getNodeName() + "'" );
		if (tNode.getNodeName().equals("sentence")){
			NamedNodeMap n = tNode.getAttributes();
			SentenceRecord s = new SentenceRecord();
			s.setId(( n.getNamedItem("id")).getNodeValue()) ;
			s.setLanguage(n.getNamedItem("language").getNodeValue());
			s.setEquiv(n.getNamedItem("equiv").getNodeValue() );
			s.setContent(tNode.getFirstChild().getNodeValue()) ;
			data.add( s );
		}
			
		if ( tNode.getFirstChild() != null ) {
			addRecord( tNode.getFirstChild() );
		}
		if ( tNode.getNextSibling() != null ) {
			addRecord( tNode.getNextSibling() );
		}
	}
		
	public boolean fromXML( Document doc ) {
		addRecord( doc );
		return true;
	}
	
	public Document toXML() {
		// TODO (only if MT should support xmlWriting)
		return null;
	}
	
	/**
	 * Returns a SentenceRecord spec'd by an ID-String (i.e. something like
	 * "DE_00048") or null, if the record's ID is not contained in the
	 * database.
	 * 
	 * @param id a unique ID-String like "EN_00012"
	 * @return Reference to matched SentenceRecord or null
	 */
	public SentenceRecord getById( String id ) {
		SentenceRecord result = null;
		Iterator it = iterator();
    
		while ( it.hasNext() && result == null ) {
			SentenceRecord tmp = (SentenceRecord) it.next();
		  	if ( tmp.getId().equals( id ) ) {
				result = tmp;
		  	}
		}
		return result;
	}
	
	/**
	 * Returns a part of the sample database as String
	 * 
	 * @param id a unique ID-String like "EN_00012"
	 * @param begin the start token 
	 * @param end the inclusive final token
	 * @return a part of the sentence from begin-end
	 */
	public String getTokensById( String id, int begin, int end ) {
		String result = new String();

		SentenceRecord t = getById( id );
		if ( t != null ) {
			MacTransTokenizer m = new MacTransTokenizer( t.getContent() );
			for ( int i = begin; i <= end; i ++ ) {
				result += m.getWord( i );
				if ( i < end ) {
					result += " ";
				}
			}
		}
		return result;
	}

	/**
	 * Returns a SentenceRecord by its ordinal position in the
	 * database or null, if the index exceeds the actual
	 * amount of records contained therein. Useful for 
	 * for-loops, for example. Note: do not rely on the record's
	 * ordinal position in an XML document; SentenceDatabase
	 * uses Hash methods, internally. Use getByID instead and
	 * pass the n'th ID!  
	 * 
	 * @param index ordinal value ranged 0 .. size() - 1
	 * @return Reference to indexed SentenceRecord or null
	 */
	public SentenceRecord getByIndex( int index ) {
		SentenceRecord result = null;
		Iterator it = iterator();
		int i = 0;
    
		while ( it.hasNext() && i <= index ) {
			SentenceRecord tmp = (SentenceRecord) it.next();
			if ( i == index ) {
				result = tmp;
			}
			i++;
		}
		return result;
	}
}
