package mactrans.database;

import java.io.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import mactrans.system.*;

/**
 * 
 * @author Iuliana Zibilianu (iuliacaz@yahoo.com)
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.3 $
 * 
 * <pre>
 * $Log: XMLDatabase.java,v $
 * Revision 1.3  2005/01/26 13:42:04  jkieneck
 * added debugging support
 *
 * Revision 1.2  2005/01/24 06:29:12  jkieneck
 * general update (too much to mention here...)
 *
 * Revision 1.1.1.1  2004/12/14 23:28:03  mactrans
 * added to repository
 *
 * </pre>
 */
public abstract class XMLDatabase extends AbstractMachineTranslationClass {
	/**
	 * Tries to import data from a specified XML document
	 *  
	 * @param doc an XML document to read data from
	 * @return true, if there were no errors during import
	 */
	public abstract boolean fromXML( Document doc );
	
	/**
	 * Returns data as an exported XML document.  
	 * 
	 * @return an XML document containing data
	 */
	public abstract Document toXML();
	
	/**
	 * loadXML tries to build an XML document from the given filename.
	 * If successful, it will pass the result to fromXML which, in return,
	 * will try to import data contained within.
	 * 
	 * @param filename a filename specifying the input source (.xml filetype)
	 * @return true, if fromXML returns true, false, if either loading or fromXML fail
	 */
	public boolean loadXML( String filename ) {
		Document document = null;
		try {
			// ---- Parse XML file ----
			DocumentBuilderFactory factory  = DocumentBuilderFactory.newInstance();
			DocumentBuilder        builder  = factory.newDocumentBuilder();
			document = builder.parse( new File( filename ) );
			// ---- Get list of nodes to given element tag name ----
		} catch( SAXParseException spe ) {
			fatalError( "XML Parsing error!" );
			fatalError( "line " + spe.getLineNumber() + " in "  + spe.getSystemId() );
			fatalError( ">>" + spe.getMessage() + "<<" );
			Exception e = ( spe.getException() != null ) ? spe.getException() : spe;
			//e.printStackTrace();
		} catch( SAXException sxe ) {
			Exception e = ( sxe.getException() != null ) ? sxe.getException() : sxe;
			e.printStackTrace();
		} catch( ParserConfigurationException pce ) {
			pce.printStackTrace();
		} catch( IOException ioe ) {
			ioe.printStackTrace();
		}
		
		boolean result = fromXML( document );
		return result;
	}
	
	/**
	 * saveXML saves data to an XML file, it uses toXML() in order to
	 * obtain an XML data representation.
	 * 
	 * @param filename a filename specifying the output source (.xml filetype)
	 * @return false, if export or file access fail
	 */
	public boolean saveXML( String filename ) {
		return false;
	}
}
